/************************************************************
*    exit_handler.c
*
*    Functions:
*        setup_exit
*        exit_signal_handler
*        shutdown
*
*************************************************************/
/************************************************************
*    Code created and written by Chris Hegarty
*    Feel free to use any portion of the code,
*    but I would appreciate credit if you use it.
*************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "exit_handler.h"

#ifndef FALSE
#define FALSE 0
#define TRUE 1
#endif
static int ran_shutdown;

/************************************************************
*    exit_signal_handler
*    Usage: exit_signal_handler()
*    Purpose: function handles abort signals and calls exit
*    Returns: Nothing
*************************************************************/

void exit_signal_handler(int signal_number){

    static int signal_counter = 0;
    signal_counter++;

    printf("\n[dtest_exit_signal_handler] received INT/TERM signal %d [%d times], exiting...\n",
			signal_number,signal_counter);

    exit(0);

}

/************************************************************
*    pshutdown
*    Usage: pshutdown()
*    Purpose: called at exit, calls clear_hook
*    Returns: Nothing
*************************************************************/

void pshutdown(void)
{
	int i;
	if(ran_shutdown == FALSE) {
        ran_shutdown = TRUE;
		clear_hook();
    }
    return;
}

/************************************************************
*    setup_exit
*    Usage: setup_exit()
*    Purpose: Setup to catch abort signals
*    Returns: Nothing
*************************************************************/

void setup_exit(void){
	ran_shutdown = FALSE;
    atexit(pshutdown);
    if(signal(SIGINT, exit_signal_handler) == SIG_ERR) {
        perror("signal(SIGINT, ...)");
    }
    if(signal(SIGTERM, exit_signal_handler) == SIG_ERR) {
        perror("signal(SIGTERM, ...)");
    }
	if(signal(SIGBREAK, exit_signal_handler) == SIG_ERR) {
        perror("signal(SIGBREAK, ...)");
    }
	if(signal(SIGABRT, exit_signal_handler) == SIG_ERR) {
        perror("signal(SIGABRT, ...)");
    }
}