#ifndef _physLib_h_
#define _physLib_h_

#include <stdio.h>
/* ----typedefs, enums, defines---------- */

#ifndef RAND_MAX
#define RAND_MAX            0xFFFF
#endif

#define MAX_GRAVITY_OBJ        20
#define MAX_RAND_X            5.0
#define MIN_RAND_X            -5.0
#define MAX_RAND_Y            5.0
#define MIN_RAND_Y            -5.0
#define MAX_RAND_Z            -3.0
#define MIN_RAND_Z            -3.0
#define MAX_RAND_RADIUS        .5
#define MIN_RAND_RADIUS        .1
#define DENSITY				20.0
#define MAX_MASS            10.0
#define ELASTICITY            1.0

#define GRAVITATION_CONSTANT    .66726
#define UNKNOWN_VELOCITY_CONSTANT .0037534

#define MAX_PARTICLES       100

typedef struct VERTEX2f{
    GLdouble x;
    GLdouble y;
}VERTEX2f;

class Vector;

class Vertex{
    public:
        GLdouble x;
        GLdouble y;
        GLdouble z;
        Vertex(void);
        Vertex(GLdouble,GLdouble,GLdouble);
        ~Vertex(void);
        Vertex operator+( Vertex );
        Vertex operator-( Vertex );
        GLdouble operator*( Vertex );
        Vertex operator*(GLdouble scalar);
        Vertex &operator=( Vector );
        GLdouble magnitude(void);
        Vertex unit(void);
};

class Vector{
    public:
        GLdouble x;
        GLdouble y;
        GLdouble z;
        GLdouble w;
        Vector(void);
        Vector(GLdouble,GLdouble,GLdouble,GLdouble);
        ~Vector(void);
        Vector operator+( Vector);
        Vector operator-( Vector);
        GLdouble operator*( Vector);
        Vector operator*(GLdouble scalar);
        Vector &operator=(Vertex);
        GLdouble magnitude(void);
        Vector unit(void);
};

class GravityObject{
    private:
        GLuint   texture_map;
        Vector acceleration;
        Vector velocity;
        Vertex position;
        GLdouble radius;
        GLdouble mass;
    public:
        Vector momentum;
        void     *particleList[MAX_PARTICLES];
        GLuint   numParticles;
        GLfloat  particleColor[4];
        GLdouble inertia;
        GLuint   displayList;

        GravityObject(void);
        ~GravityObject(void); 
        void SetAcceleration(Vector);
        Vector GetAcceleration(void);
        void SetVelocity(Vector);
        Vector GetVelocity(void);
        void SetPosition(Vertex);
        Vertex GetPosition(void);
        void SetRadius(GLdouble);
        GLdouble GetRadius(void);
        void SetMass(GLdouble);
        GLdouble GetMass(void);
};

/* ---------- Globals --------- */

/* ---------- Prototypes ------- */
GravityObject *createGravityObj(void);

void getColor(float *);
Vector calculateAcceleration(Vector,GLdouble,GLdouble);
Vector calculateGravityForce(GravityObject *, GravityObject *);
Vector calculateVelocity(Vector, Vector, GLdouble);
Vertex calculatePosition(Vector, Vertex, GLdouble);
Vector calculateCollision(GravityObject *, GravityObject *);
Vector calculateMomentum(GravityObject *);

GLuint createBackground(char *);

#endif
